#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
ext_pylib.meta
~~~~~~~~~~~~~~

Classes and functions for metaprogramming
"""

from __future__ import absolute_import

from .meta import DynamicProperty, setdynattr



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
ext_pylib.user
~~~~~~~~~~~~~~

Extra python libraries for scaffolding server scripts.
"""

from __future__ import absolute_import

from .user import get_current_username, get_current_groupname



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
ext_pylib.input
~~~~~~~~~~~~~~~~

Functions for displaying and handling input on the terminal.
"""

from __future__ import absolute_import

# Use Python 2 input unless raw_input doesn't exist
try:
    INPUT = raw_input
except NameError:
    INPUT = input

# pylint: disable=wrong-import-position
# this import MUST be after INPUT is defined
from .prompts import prompt, prompt_str, warn_prompt



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
ext_pylib.files
~~~~~~~~~~~~~~~

Class objects for files and directories.
"""

from __future__ import absolute_import

from .dir import Dir
from .file import File, Section, SectionFile, Template, TemplateFile, Parsable, ParsableFile



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# name:             dir.py
# author:           Harold Bradley III
# email:            harold@bradleystudio.net
# created on:       11/03/2015
#
# pylint:           disable=line-too-long

"""
ext_pylib.files.dir
~~~~~~~~~~~~~~~~~~~

A class that describes a directory and gives functions to create the directory.
It extends Node.
"""

from __future__ import absolute_import, print_function, unicode_literals

import os
import shutil

from .node import Node
from ..input import prompt


def copytree(source, destination, symlinks=False, ignore=None):
    """A wrappper around copytree that allows copying into an existing directory.
    Adapted from: http://stackoverflow.com/questions/1868714."""
    for item in os.listdir(source):
        item_src = os.path.join(source, item)
        item_dst = os.path.join(destination, item)
        if os.path.isdir(item_src): # It's a directory
            if os.path.exists(item_dst): # It's a directory that already exists
                copytree(item_src, item_dst)
            else:
                shutil.copytree(item_src, item_dst, symlinks, ignore)
            return True
        else: # It's a file
            if os.path.exists(item_dst):
                if not prompt(item_dst + ' already exists. Replace with ' + item_src + '?'):
                    print('Skipping.')
                    return True
                os.remove(item_dst) # It's a file that already exists; remove it, then copy
            shutil.copy2(item_src, item_dst)
            return True


class Dir(Node):
    """A class that describes a directory node. Extends Node class.
    See Node class for atts to pass in at init.

    :param atts: See notes in node.py

    Usage::

        >>> from ext_pylib.files import Dir

        >>> a_dir = Dir({'path' : '/the/path/', 'perms' : 0o600, 'owner' : 'root', 'group' : 'root'})
        >>> a_dir.path
        '/the/path/'

        >>> a_dir.create()
        Creating directory "/the/path/"... [OK]
    """

    def __str__(self):
        """Returns a string with the path."""
        if not self.path:
            return '<files.Dir:stub>'
        return self.path

    def create(self, fill_with=None):  # pylint: disable=arguments-differ
        """Creates the directory structure."""
        if self.exists():
            print(self.path + ' already exists.')
            return True
        print('Creating directory "' + self.path + '"...', end=' ')
        try:
            os.makedirs(self.path)
            print('[OK]')
        except Exception as error: # pylint: disable=broad-except
            print('[ERROR]')
            print(error)
            return False
        if fill_with:
            self.fill(fill_with)
        return all([self.chmod(), self.chown()])

    def remove(self, ask=True): # pylint: disable=arguments-differ
        """Removes the directory structure."""
        if not self.path:
            return True
        if not self.exists():
            print(self.path + ' doesn\'t exist.')
            return True
        if not ask or prompt('Completely remove ' + self.path + ' and all containing files and folders?'):
            print('Removing "' + self.path + '"...', end=' ')
            try:
                shutil.rmtree(self.path)
                print('[OK]')
                return True
            except Exception as error: # pylint: disable=broad-except
                print('[ERROR]')
                print(error)
                return False

    def fill(self, fill_with):
        """Fills the directory with the contents of "fill_with" (another Dir instance)."""
        if not self.exists():
            print('Copy failed. [ERROR]')
            raise IOError(self.path + 'does not exist')
        if not fill_with.exists(): # Requires a Dir object, NOT a string
            print('Copy failed. [ERROR]')
            raise IOError(fill_with + 'does not exist')
        print('Filling "' + self.path + '" with contents of "' + fill_with + '"...')
        try:
            copytree(fill_with.path, self.path) # copytree(source, destination)
            print('Copy complete. [OK]')
            return True
        except Exception as error:  # pylint: disable=broad-except
            print('Copy failed. [ERROR]')
            print(error)
            return False

    # pylint: disable=no-member
    @Node.path.setter
    def path(self, path):
        """Sets the path. A Dir path must end in '/'.
        If it doesn't end in '/', one is appended automatically."""
        if path is None:
            return
        # Force directory to end in '/'
        if not path.endswith('/'):
            path = path + '/'
        Node.path.fset(self, path)

    @property
    def parent_dir(self):
        """Returns a Dir instance representing the parent directory."""
        return Dir(self.parent_node.get_atts())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# name:             meta.py
# author:           Harold Bradley III
# email:            harold@bradleystudio.net
# created on:       01/28/2016
#

"""
ext_pylib.meta.meta
~~~~~~~~~~~~~~~~~~~

Classes and functions for meta-programming with python.
This is mostly experimental stuff. For now, class DynamicProperty is merely a
close reimplementation of python's native property().

Credit for much of this goes to:
    http://eev.ee/blog/2012/05/23/python-faq-descriptors/
"""


class DynamicProperty(object):
    """A re-implementation of python's native property().
    Most of this code derived from: http://eev.ee/blog/2012/05/23/python-faq-descriptors/.

    :param getter: The getter function of the property.

    Usage::

        >>> from ext_pylib.meta import DynamicProperty

        >>> def getter_func(self):
        >>>     return self._property

        >>> def setter_func(self, value):
        >>>     self._property = value

        >>> class Cls(object): pass
        >>> instance = Cls()
        >>> instance.__class__.new_property = DynamicProperty(getter_func)
        >>> instance.__class__.new_property = \
                instance.__class__.new_propety.create_setter(setter_func)

        >>> instance.new_property = 'value'
        >>> instance.new_property
        'value'
    """

    def __init__(self, getter=None):
        """Initializes the DynamicProperty's getter function."""
        self.getter = getter

    def __get__(self, instance, owner):
        """Returns the result of running the getter function.
        This is a wrapper for the property's actual getter function."""
        self.owner = owner  # Not used pylint: disable=attribute-defined-outside-init
        if instance is None:
            return self
        return self.getter(instance)

    def __set__(self, instance, value):
        """Calls the setter function.
        This is a wrapper for the property's actual setter function."""
        self.setter(instance, value)

    def create_setter(self, setter):
        """Sets the setter function to the passed arg setter (a function).
        Returns self. (Composite Pattern)."""
        self.setter = setter
        return self

    @staticmethod
    def setter(*args, **kwargs):  # pylint: disable=method-hidden,unused-argument
        """A stub. If it isn't overwritten, the property is read-only."""
        raise TypeError("Cannot modify property.  It doesn't have a setter function.")

def setdynattr(obj, attribute, getter_func=None, setter_func=None):
    """Creates a dynamic property on an object using DynamicProperty class."""
    if not getter_func:
        def getter_func(self):  # pylint: disable=function-redefined
            """Default getter function.
            Uses '_' + attribute name as a private variable to hold the
            property's value."""
            return getattr(self, '_' + attribute, None)

    if not setter_func:
        def setter_func(self, value): # pylint: disable=function-redefined
            """Default setter function.
            Uses '_' + attribute name as a private variable to hold the
            property's value."""
            return setattr(self, '_' + attribute, value)

    setattr(obj.__class__, attribute, DynamicProperty(getter_func))
    prop = getattr(obj.__class__, attribute)
    prop = prop.create_setter(setter_func)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
ext_pylib.terminal
~~~~~~~~~~~~~~~~~~

Functions for coloring and managing the terminal.
"""

from __future__ import absolute_import

from . import ansi
from . import colors



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# name:             ansi.py
# author:           Harold Bradley III
# email:            harold@bradleystudio.net
# created on:       02/23/2016
#

"""
ext_pylib.terminal.ansi
~~~~~~~~~~~~~~~~~~~~~~~

Ansi escape codes and functions.

Most of these functions write their results directly to standard out unless
passed get_string=True, in which case they just return the escape as a string.
"""

from sys import stdout


# GENERAL ASCII CODES

NULL = '\00' # Null character
BEL = '\007' # Terminal Bell
BS = '\010'  # Backspace
HT = '\011'  # Horizontal Tab
LF = '\012'  # Linefeed (newline)
VT = '\013'  # Vertical Tab
FF = '\014'  # Formfeed (or NP: new page)
CR = '\015'  # Carriage Return
ESC = '\033' # Escape character
DEL = '\177' # Delete character


# ESCAPE SEQUENCES

CSI = '\033['
OSC = '\033]'
DELIMITER = ';'

def attributes(*args):
    """Returns multiple attributes concatenated and separated by DELIMITER."""
    return reduce(lambda a, b: str(a) + DELIMITER + str(b), args)

def escape(escape, get_string):
    """Prints an escape sequence or returns it as a string."""
    if get_string:
        return escape
    stdout.write(escape)
    stdout.flush()


    ## Cursor Handling ##

def cursor_hide(get_string=False):
    """Hides the cursor."""
    return escape(ansi.CSI + '?25l', get_string)

def cursor_show(get_string=False):
    """Shows the cursor."""
    return escape(ansi.CSI + '?25h', get_string)

def cursor_up(n=1, get_string=False):
    """Moves your cursor up 'n' cells."""
    return escape(CSI + str(n) + 'A', get_string)

def cursor_down(n=1, get_string=False):
    """Moves your cursor down 'n' cells."""
    return escape(CSI + str(n) + 'B', get_string)

def cursor_right(n=1, get_string=False):
    """Moves your cursor right (forward) 'n' cells."""
    return escape(CSI + str(n) + 'C', get_string)

def cursor_left(n=1, get_string=False):
    """Moves your cursor left (backward) 'n' cells."""
    return escape(CSI + str(n) + 'D', get_string)

def cursor_next_line(n=1, get_string=False):
    """Moves your cursor to the next 'n' lines."""
    return escape(CSI + str(n) + 'F', get_string)

def cursor_previous_line(n=1, get_string=False):
    """Moves your cursor to the previous 'n' lines."""
    return escape(CSI + str(n) + 'G', get_string)

def cursor_horizontal_absolute(n=1, get_string=False):
    """Moves your cursor to the 'n' column."""
    return escape(CSI + str(n) + 'G', get_string)

def cursor_position(x=0, y=0, get_string=False):
    """Moves youescape = cursor to position (x, y).
    NOTE: This assumes starting at (0, 0) which is different than the ANSI
    standard; it also assumes (x, y) and not (y, x) per ANSI standard."""
    return escape(CSI + attributes(y+1, x+1) + 'f', get_string)

def cursor_save(get_string=False):
    """Save the current cursor position."""
    return escape(CSI + 's', get_string)

def cursor_restore(get_string=False):
    """Restore the last saved cursor position."""
    return escape(CSI + 'u', get_string)

def attributes_save(get_string=False):
    """Save the current cursor position and attributes."""
    return escape(ESC + '7', get_string)

def attributes_restore(get_string=False):
    """Restore the last saved cursor position and attributes."""
    return escape(ESC + '8', get_string)

def get_cursor_pos():
    """Returns a tuple of (x, y) of current cursor position.
    Note this follows conventional (x, y) order and starts with (0, 0) and not
    the order according to the ANSI standard."""
    QCU = CSI + '6n'  # Query cursor position (to stdin)
    RCU = CSI + '{0};{1}R' # Reports Cursor positon (result from above query
                           # Reports as <ESC>[{row};{column}R
                           # Note that this is backwards order


    ## Clearing the screen ##

def clear(get_string=False):
    """Clears the entire screen."""
    return escape(CSI + '2J', get_string)

def clear_down(get_string=False):
    """Clears the screen from the cursor down."""
    return escape(CSI + '0J', get_string)

def clear_up(get_string=False):
    """Clears the screen from the cursor down."""
    return escape(CSI + '1J', get_string)

def clear_line(get_string=False):
    """Clears the entire line."""
    return escape(CSI + '2K', get_string)

def clear_line_forward(get_string=False):
    """Clears the entire line."""
    return escape(CSI + '0K', get_string)

def clear_line_back(get_string=False):
    """Clears the entire line."""
    return escape(CSI + '1K', get_string)

def reset(get_string=False):
    """Clears the entire screen and places cursor at top left corner."""
    return escape(clear(get_string=True) + cursor_position(0, 0, get_string=True), get_string)


    ## Terminal Settings ##

def reset_terminal(get_string=False):
    """Resets the terminal."""
    return escape(ESC + 'c', get_string)

def enable_line_wrap(get_string=False):
    """Enables line wrapping."""
    return escape(CSI + '7h', get_string)

def disable_line_wrap(get_string=False):
    """Disables line wrapping."""
    return escape(CSI + '7h', get_string)

def set_scroll_all(get_string=False):
    """Enable scrolling for the entire screen."""
    return escape(CSI + 'r', get_string)

def set_scroll(start_row, end_row, get_string=False):
    """Enable scrolling from start_row to end_row."""
    return escape(CSI + attributes(start_row, end_row) + 'r', get_string)

def scroll_up(get_string=False):
    """Scrolls up."""
    return escape(ESC + 'D', get_string)

def scroll_down(get_string=False):
    """Scrolls down."""
    return escape(ESC + 'M', get_string)


    ## SGR Escape Sequences ##

def sgr(*args):
    """Returns an SGR (Select Graphic Rendition) escape sequence given one or
    more attributes."""
    return CSI + attributes(*args) + 'm'

RESET = '0'
BOLD = '1'
UNDERLINE = '4'
BLINK = '5'
RBLINK = '6'
REVERSE = '7'
CONCEAL = '8'

BLACK = '30'
RED = '31'
GREEN = '32'
YELLOW = '33'
BLUE = '34'
MAGENTA = '35'
CYAN = '36'
WHITE = '37'
EXTENDED = '38'

BG_BLACK = '40'
BG_RED = '41'
BG_GREEN = '42'
BG_YELLOW = '43'
BG_BLUE = '44'
BG_MAGENTA = '45'
BG_CYAN = '46'
BG_WHITE = '47'
BG_EXTENDED = '48'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
ext_pylib.domain
~~~~~~~~~~~~~~~~

Classes and functions for manipulating domains.
"""

from __future__ import absolute_import

from .domain import Domain, get_server_ip



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# name:             user.py
# author:           Harold Bradley III
# email:            harold@bradleystudio.net
# created on:       01/13/2016
#

"""
ext_pylib.user.user
~~~~~~~~~~~~~~~~~~~

Functions for managing users.
"""

import grp
import os
import pwd

def get_current_username():
    """Returns the current username as a string."""
    return pwd.getpwuid(os.getuid()).pw_name

def get_current_groupname():
    """Returns the current groupname as a string."""
    return grp.getgrgid(os.getgid()).gr_name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# name:             node.py
# author:           Harold Bradley III
# email:            harold@bradleystudio.net
# created on:       11/03/2015
#
# pylint:           disable=line-too-long

"""
ext_pylib.files.node
~~~~~~~~~~~~~~~~~~~~

An abstract class that describes a node to be extended by a directory and file
class respectively.
"""

from __future__ import absolute_import, print_function, unicode_literals

import grp
import os
import pwd

from ..user import get_current_username, get_current_groupname


class Node(object):
    """An abstract class representing a node object.

    This class is intended primarily as a wrapper for directory and file
    management and is designed to be extended by file and directory classes.

    Node is initialized with a dict of attributes. Attributes that aren't given
    are just initialized as None. If a path isn't given, the node is set to
    path = None. This effectively makes the Node a "Stub", in which methods do
    nothing but return True (except exists()) enabling a graceful fail.

    :param atts['path']: The path (string) of the node.
    :param atts['perms']: The permissions (int or octal) of the node.
    :param atts['owner']: The owner (string) of the node.
    :param atts['group']: The group (string) of the node.

    * Note: This class is intended to be subclassed.

    Usage::

        >>> from ext_pylib.files.node import Node

        >>> node = Node({'path' : '/the/path/', 'perms' : 0o600, 'owner' : 'root', 'group' : 'root'})
        >>> node.path
        '/the/path/'
    """

    def __init__(self, atts=None):
        """Initializes a new Node instance."""
        atts = atts or {}
        if 'path' not in atts:
            self.path = None # !! Make sure path is at least initialized
        for attribute in atts:
            setattr(self, attribute, atts[attribute])

    def __str__(self):
        """Returns a string with the path."""
        if not self.path:
            return '<files.Node:stub>'
        return self.path

    def __repr__(self):
        """Returns a python string that evaluates to the object instance."""
        return "{0}({1})".format(self.__class__.__name__, self.get_atts(string=True))

    def __add__(self, other):
        """Allows string concatenation with the path."""
        return str(self) + other

    def __radd__(self, other):
        """Allows string concatenation with the path."""
        return other + str(self)

    def get_atts(self, string=False):
        """Returns a python string of attributes (as a dict) used to create this object."""
        if not string:
            return {'path' : self.path, 'perms' : getattr(self, '_perms', None), 'owner' : self.owner, 'group' : self.group}
        # Note that any atts added later must be added here for this to work.
        atts = "{'path' : "
        if self.path:
            atts += "'" + self.path + "', "
        else:
            atts += 'None, '

        atts += "'perms' : "
        if self.perms:
            atts += self.perms_as_string() + ", "
        else:
            atts += 'None, '

        atts += "'owner' : "
        if self.owner:
            atts += "'" + self.owner + "', "
        else:
            atts += 'None, '

        atts += "'group' : "
        if self.group:
            atts += "'" + self.group + "'}"
        else:
            atts += 'None}'

        return atts

    @staticmethod
    def create(*args, **kwargs):
        """A stub to be implemented by child class."""
        raise NotImplementedError('[ERROR] Cannot call method on file.Node. It is an abstract class.')

    @staticmethod
    def remove(*args, **kwargs):
        """A stub to be implemented by child class."""
        raise NotImplementedError('[ERROR] Cannot call method on file.Node. It is an abstract class.')

    def chmod(self, perms=None):
        """Sets the permissions on the file/directory."""
        if not self.path:
            return True
        if not self.exists():
            raise IOError(self.path + ' does not exist. Cannot set owner and permissions. [!]')
        if not perms:
            perms = self.perms
        if not perms:
            return True
        print('Setting permissions on ' + self.path + ' to "' + \
              self.perms_as_string(perms)  + '"...', end=' ')
        try:
            os.chmod(self.path, perms) # Be sure to use leading '0' as chmod takes an octal
            print('[OK]')
            return True
        except Exception as error:  # pylint: disable=broad-except
            print('[FAILED]')
            print(error)
            return False

    def chown(self, owner=None, group=None):
        """Sets the owner and group of the directory."""
        if not self.path:
            return True
        if not self.exists():
            raise IOError(self.path + ' does not exist. Cannot set owner and permissions. [!]')
        if not owner:
            owner = self.owner
            if not owner:
                owner = get_current_username()
        if not group:
            group = self.group
            if not group:
                group = 'nogroup' if owner == 'nobody' else get_current_groupname()
        print('Setting owner on ' + self.path + ' to "' + owner  + ':' + group + '"...', end=' ')
        try:
            uid = pwd.getpwnam(owner).pw_uid
            gid = grp.getgrnam(group).gr_gid
            os.chown(self.path, uid, gid)
            print('[OK]')
            return True
        except Exception as error:  # pylint: disable=broad-except
            print('[FAILED]')
            print(error)
            return False

    def exists(self):
        """Returns true if this directory exists on disk."""
        if not self.path:
            return False
        if os.path.exists(self.path):
            return True
        return False

    def verify(self, repair=False):
        """Verifies the existence, permissions, ownership, and group of the file/directory."""
        if not self.path:
            return True
        print('')
        print('Checking "' + self.path + '"...', end=' ')
        if not self.exists():
            print('[WARN]')
            print('[!] ' + self.path + ' doesn\'t exist')
            if not repair:
                return False
            self.create()
            return self.verify(repair)
        print('[OK]')

        # Assume the checks pass
        perms_check = owner_check = group_check = True

        if self.perms:
            print('--> Checking permissions for ' + self.path,)
            perms_check = self.perms == self.actual_perms
            print(' (should be: ' + self.perms_as_string() + ', actual: ' + \
                self.perms_as_string(self.actual_perms) + ')', end=' ')
            print('[OK]' if perms_check else '[WARN]')
            if not perms_check and repair:
                perms_check = self.chmod()
                return self.verify(repair)

        if self.owner:
            print('--> Checking owner for ' + self.path,)
            owner_check = self.owner == self.actual_owner
            print(' (should be: ' + self.owner + ', actual: ' +
                  self.actual_owner + ')', end=' ')
            print('[OK]' if owner_check else '[WARN]')

        if self.group:
            print('--> Checking group for ' + self.path,)
            group_check = self.group == self.actual_group
            print(' (should be: ' + self.group + ', actual: ' + self.actual_group + ')', end=' ')
            print('[OK]' if group_check else '[WARN]')
            if not (group_check or owner_check) and repair:
                group_check = owner_check = self.chown()
                return self.verify(repair)

        result = perms_check and owner_check and group_check

        if not result:
            print('[ERROR] Verification for "' + self.path + '" failed.')

        return result

    def repair(self):
        """Runs verify() with the repair flag set."""
        return self.verify(True)

    @property
    def path(self):
        """Returns the path, if it exists."""
        return getattr(self, '_path', None)

    @path.setter
    def path(self, path):
        """Validates, then sets the path."""
        # pylint: disable=attribute-defined-outside-init
        # Check for None
        if path is None:
            print('[Notice] file.Node was initialized with an empty path.  Continuing as a stub.')
            self._path = None
            return
        # Check for empty string
        if path == '':
            raise ValueError('"path" cannot be set to an empty string in an file.Node class.')
        # Check for valid characters
        for char in path:
            if char not in '-_.() abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789/':
                raise ValueError('"' + path + '" is not allowed as an file.Node.')
        # Clean path of extra slashes
        while "//" in path:
            path = path.replace('//', '/')
        self._path = path

    @property
    def name(self):
        """Returns just the name (portion of the path) as a string."""
        path = self.path
        if not path:
            return None
        if path.endswith('/'):  # removes any trailing '/'
            path = path[:-1]
        return path.rsplit('/')[-1]  # last element after splitting based on '/'

    @property
    def parent_node(self):
        """Returns the parent node as a Node object (usually the parent directory)."""
        if not self.path:
            return None
        if self.path == '/':  # '/' has no parent
            return None

        path = self.path
        if not path.endswith('/'):  # makes the spliting easier to have '/' at the end
            path = path + '/'

        if path.endswith('/../'):  # just keep adding /../.. etc... to get parent of relative path.
            parent_path = path + '..'
        elif path.count('/') == 1 and path.endswith('/'):
            # parent of relative path (has only one '/' and ends with it) is <path>/..
            parent_path = path + '..'
        else:
            parent_path = path.rsplit('/', 2)[0] + '/'

        return Node({'path' : parent_path})

    @property
    def actual_perms(self):
        """Returns the perms as it is on disk."""
        if not self.path:
            return None
        return os.stat(self.path).st_mode & 511

    @property
    def perms(self):
        """Returns the perms (oct/int)."""
        return getattr(self, '_perms', None)

    @perms.setter
    def perms(self, perms):
        """Sets the perms (oct/int)."""
        # pylint: disable=attribute-defined-outside-init
        if not perms:
            self._perms = None
            return
        try:
            perms = int(perms)
        except ValueError as error:
            print(error)
            print('[ERROR] ' + perms + ' must be set to an int.')
            raise
        if not 0 <= perms <= 511:
            raise ValueError('"perms" cannot be set to ' + self.perms_as_string(perms) + '.')
        self._perms = perms

    def perms_as_string(self, perms=None):
        """Returns the perms as a string."""
        if not perms:
            if not self.perms:
                return 'None'
            return format(self.perms, '#o')
        return format(perms, '#o')

    @property
    def actual_owner(self):
        """Returns the owner (string) as it is on disk."""
        if not self.path:
            return None
        return pwd.getpwuid(os.stat(self.path).st_uid).pw_name

    @property
    def owner(self):
        """Returns the owner (string)."""
        return getattr(self, '_owner', None)

    @owner.setter
    def owner(self, owner):
        """Sets the owner (string)."""
        # pylint: disable=attribute-defined-outside-init
        if owner is None:
            owner = get_current_username()
        else:
            try:
                # Make sure this is a valid user
                uid = pwd.getpwnam(owner)  # pylint: disable=unused-variable
            except KeyError:
                print('[ERROR] ' + owner + ' is not a valid user.')
                raise
        self._owner = owner

    @property
    def actual_group(self):
        """Returns the group (string) as it is on disk."""
        if not self.path:
            return None
        return grp.getgrgid(os.stat(self.path).st_gid).gr_name

    @property
    def group(self):
        """Returns the group (string)."""
        return getattr(self, '_group', None)

    @group.setter
    def group(self, group):
        """Sets the group (string)."""
        # pylint: disable=attribute-defined-outside-init
        # Check for empty string
        if group is None:
            group = get_current_groupname()
        else:
            try:
                # Make sure this is a valid group
                gid = grp.getgrnam(group) # pylint: disable=unused-variable
            except KeyError:
                print('[ERROR] ' + group + ' is not a valid group.')
                raise
        self._group = group



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# name:             file.py
# author:           Harold Bradley III
# email:            harold@bradleystudio.net
# created on:       11/04/2015
#
# pylint:           disable=no-member,line-too-long

"""
ext_pylib.files.file
~~~~~~~~~~~~~~~~~~~~

A class to manage and create files. Also includes three mixin classes Parsable,
Section, and Template.
"""

from __future__ import absolute_import, print_function, unicode_literals

import os
import re

from .dir import Dir
from .node import Node
from ..input import prompt
from ..meta import setdynattr


class File(Node):
    """An class to manage a file's permissions, ownership, and path. Extends Node.
    See Node class for atts to pass in at init.

    The Section mixin adds methods useful for processing template section
    files. A section file is a template of a configuration file that only
    represents a particular section of that file. It begins and ends with a
    delineator (for example: ## START:SECTION_NAME ## and ## END:SECTION_NAME
    ##). A use case would be how WordPress delineates a particular section of
    the htaccess file in its root directory with a start line and an end line.
    This is a section of the full htaccess file and could be managed by a
    Section mixin.

    The Template mixin adds a method useful for processing a regular template
    file: apply_using(). It assumes that the file contains placeholder text to
    be replaced by actual data. The placeholders and actual data are passsed
    into the method as a dict. The resulting data is returned (presumably to be
    saved in another file.)

    The Parsable mixin adds a method useful for parsing (presumably)
    configuration files. It takes a dict of attribute names and regexes to be
    used. When setup_parsing() is called, a dynamic property is created for
    getting and setting a value in self.data based on the regex.

    :param atts: See notes in node.py

    Usage::

        >>> from ext_pylib.files import File

        >>> a_file = File({'path' : '/the/path/file', 'perms' : 0o600, 'owner' : 'root', 'group' : 'root'})
        >>> a_file.path
        '/the/path/file'

        >>> a_file.read()
        'The data...
    """

    def __init__(self, atts=None):
        """Initializes a new File instance."""
        super(File, self).__init__(atts)
        self.data = '' # Initialize data as an empty string.

    def __str__(self):
        """Returns a string with the path."""
        if not self.path:
            return '<file.File:stub>'
        return self.path

    def create(self, data=None):  # pylint: disable=arguments-differ
        """Creates the file/directory."""
        # pylint: disable=attribute-defined-outside-init
        if not self.path: # For stubs, just return True
            return True
        if self.exists():
            print(self.path + ' already exists.')
            if not prompt('Replace it?', False):
                return False
        print('Creating ' + self.path + '... ', end=' ')

        # Create parent directories
        if not self.parent_dir.exists():
            try:
                print('')
                self.parent_dir.create()
            except Exception as error:  # pylint: disable=broad-except
                print('[ERROR]')
                print(error)

        # Create the file
        try:
            file_handle = open(self.path, 'w')
            if data:  # If data was passed or data exists, write it.
                self.data = data
            if getattr(self, 'data', None):
                self.write(self.data, False, file_handle)
            file_handle.close()
            print('[OK]')
        except Exception as error:  # pylint: disable=broad-except
            print('[ERROR]')
            print(error)
            return False
        return all([self.chmod(), self.chown()])

    def remove(self, ask=True):  # pylint: disable=arguments-differ
        """Removes the file/directory."""
        if not self.path:
            return True
        if not self.exists():
            print(self.path + ' doesn\'t exist.')
            return True
        if not ask or prompt('Remove ' + self.path + '?'):
            os.remove(self.path)
            return True

    def read(self, flush_memory=False):
        """Returns the contents of the file.
           If the file doesn't exist, returns an empty string.

           Note that method first attempts to return the contents as in memory
           (which might differ from what is on disk)."""
        if flush_memory:  # Empty memory to force reading from disk
            self.data = ''
        if self.data != '':
            return self.data
        if not self.exists():  # If no data in memory and doesn't exist,
            self.data = ''     # return an empty string.
            return self.data
        try:  # Otherwise, try to read the file
            file_handle = open(self.path, 'r')
            self.data = file_handle.read()
            file_handle.close()
            return self.data
        except Exception:  # pylint: disable=broad-except
            print('[ERROR]')
            raise

    def readlines(self):
        """Returns the contents of the file as a list for iteration."""
        return self.read().split('\n')

    def write(self, data=None, append=True, handle=None):
        """Writes data to the file."""
        # pylint: disable=attribute-defined-outside-init
        if data:
            self.data = data  # Keep the data we're saving in memory.
        else:
            if self.data == '':
                raise UnboundLocalError('Must pass data to write method of File class.')
            else:
                data = self.data

        try:
            if handle: # When passed a handle, rely on the caller to open.close the file
                file_handle = handle
                file_handle.write(data)
            else:
                flags = 'a' if append else 'w'
                file_handle = open(self.path, flags)
                file_handle.write(data)
                file_handle.close()
            return True
        except Exception:  # pylint: disable=broad-except
            print('[ERROR]')
            return False

    def append(self, data, handle=None):
        """Appends the file with data. Just a wrapper."""
        return self.write(data, True, handle)

    def overwrite(self, data, handle=None):
        """Overwrites the file with data. Just a wrapper."""
        return self.write(data, False, handle)

    @Node.path.setter
    def path(self, path):
        """Sets the path."""
        # Check for None
        if path is None:
            return
        # File cannot end in '/'
        if path.endswith('/'):
            raise ValueError('"path" cannot end in "/" in a file.File class.')
        Node.path.fset(self, path)

    @property
    def parent_dir(self):
        """Returns a Dir instance representing the parent directory."""
        return Dir(self.parent_node.get_atts())


class Section(object):
    """A mixin class to work with a section template file.
    See Node class for atts to pass in at init.

    :param atts: See notes in node.py

    Usage::

        >>> from ext_pylib.files import File, Section

        >>> class SectionFile(Section, File): pass

        >>> a_file = SectionFile({'path' : '/the/path/file', 'perms' : 0o600, 'owner' : 'root', 'group' : 'root'})
        >>> a_file.is_applied(File({'path' : '/another/path/file'}).read())
        True
    """

    def is_applied(self, data):
        """Returns true if data has this section applied exactly."""
        return self.read() in data

    def is_in(self, data):
        """Returns true if data has the section, whether or not it is applied
        exactly."""
        # pylint: disable=attribute-defined-outside-init
        self._start_pos = data.find(self.start_section)
        self._end_pos = data.find(self.end_section)
        if self._start_pos < 0 and self._end_pos < 0:
            return False
        elif self._start_pos < self._end_pos:
            return True
        else:
            raise ValueError('Data passed to is_in() not formatted properly.')

    def apply_to(self, data, overwrite=False):
        """Returns a string in which the section is applied to the data."""
        if self.is_applied(data):
            return data
        if self.is_in(data):
            if overwrite:
                return data[:self._start_pos] + self.read() + '\n' + \
                        data[self._end_pos + len(self.end_section) + 1:]
            else:
                raise ValueError('[WARN] Section already exists, but overwrite flag was not set.')
        else:
            return data + '\n' + self.read() + '\n'

    @property
    def start_section(self):
        """Returns the string that denotes the start of the section."""
        if self.read() == '':
            raise EOFError('Section file has no data')
        return self.readlines()[0]

    @property
    def end_section(self):
        """Returns the string that denotes the end of the section."""
        if self.read() == '':
            raise EOFError('Section file has no data')
        lines = self.readlines()
        if len(lines) < 2:
            raise ValueError('Not a valid section file.')
        if lines[-1] != '':  # If the last line is blank, use the line before it.
            return lines[-1]
        return lines[-2]


class SectionFile(Section, File):
    """A File class implementing the Section Mixin."""


class Template(object):  # pylint: disable=too-few-public-methods
    """A mixin to work with a template file with placeholders.
    See Node class for atts to pass in at init.

    :param atts: See notes in node.py

    Usage::

        >>> from ext_pylib.files import File, Template

        >>> class TemplateFile(Template, File): pass

        >>> a_file = File({'path' : '/the/path/file', 'perms' : 0o600, 'owner' : 'root', 'group' : 'root'})
        >>> a_file.apply_using({'placeholder' : 'value'})
        The data...
    """

    def apply_using(self, placeholders):
        """Returns a string with placeholders replaced.
        Takes a dict of placeholders and values to replace."""
        data = self.read() # temp, throw-away (after returning) data value
        for placeholder, value in list(placeholders.items()):
            data = data.replace(placeholder, value)
        return data


class TemplateFile(Template, File):
    """A File class implementing the Template Mixin."""


class Parsable(object):
    """A mixin to be used with a File class to allow parsing.
    See Node class for atts to pass in at init.

    :param atts: See notes in node.py

    Usage::

        >>> from ext_pylib.files import File, Parsable

        >>> class ParsableFile(Parsable, File): pass

        >>> a_file = File({'path' : '/the/path/file', 'perms' : 0o600, 'owner' : 'root', 'group' : 'root'})
        >>> a_file.setup_parsing('htdocs' : 'DocumentRoot (.*)')

        >>> a_file.htdocs
        'example.com'
    """

    def setup_parsing(self, regexes=None):
        """Takes a dict of name:regex to parse self.data with.
           regex is either a string, a tuple of one, or a tuple of two with the
           second element being the regex mask used for assigning a new value
           to this property. It must contain '{}' to be the marker for the
           placeholder of the new value."""
        if not regexes:
            regexes = self.regexes
        for attribute, regex in regexes.items():
            att = getattr(self.__class__, attribute, None)
            if hasattr(self, attribute) and \
                not att.__class__.__name__ == 'DynamicProperty':
                raise AttributeError('Attribute "' + attribute + \
                                     '" already in use.')
            self.create_parseable_attr(attribute, regex)

    def create_parseable_attr(self, attribute, regex_tuple):
        """Creates a dynamic attribure on the Parsable class.
        This dynamically creates a property with a getter and setter. The regex
        is a closure. Each time the attribute is accessed, the regex is run
        against the data in memory. When the attribute is set to a new value,
        this value is changed in memory. file.write() must be called to write
        the changes to memory.

        NOTE: Because these are dynamic properties they are on the Class NOT
        the instance. This can cause difficult-to-find bugs when using the
        class with mulitple regexes and the same named attributes. Be sure to
        call setup_parsing() every time you change these regexes (expecially
        when changing to a regex/mask tuple or just a regex string).
        """
        if isinstance(regex_tuple, tuple):
            if len(regex_tuple) == 2:
                regex, mask = regex_tuple
            else:
                regex, mask = regex_tuple[0], '{}'
        else:
            regex, mask = regex_tuple, '{}'
        def getter_func(self):
            """Parsable dynamic attribute getter function."""
            results = re.findall(regex, self.read())
            if not results:
                return None
            elif len(results) == 1:
                return results[0]
            return results

        def setter_func(self, value):
            """Parsable dynamic attribute setter function.
            Note that this is only changing the value in memory.  You must call
            write()."""
            if not re.findall(regex, self.read()):
                # If the value doesn't exist, add it to the end of data
                self.data = self.data + '\n' + mask.format(value)
            else:  # otherwise just change it everywhere it exists
                self.data = re.sub(regex, mask.format(value), self.read())

        setdynattr(self, attribute, getter_func, setter_func)


class ParsableFile(Parsable, File):
    """A File class implementing the Parsable Mixin."""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# name:             domain.py
# author:           Harold Bradley III
# email:            harold@bradleystudio.net
# created on:       11/03/2015
#
# pylint:           disable=invalid-name,attribute-defined-outside-init
#                   (ip is perfectly valid)
#
#                   TODO: add DNS API capabilities

"""
ext_pylib.domain.domain
~~~~~~~~~~~~~~~~~~~~~~~

A class to describe and manage a domain name and it's corresponding ip address.
"""

from __future__ import print_function

import re
import socket

try:
    import requests
except ImportError:
    raise ImportError("ext_pylib's domain module requires module requests.")


def get_server_ip(get_ip_urls=None):
    """Returns the IP of this server."""
    get_ip_urls = get_ip_urls or ['http://techterminal.net/myip/',]
    for url in get_ip_urls:
        ip = requests.get(url).text
        if not ip == '127.0.0.1':
            return ip
    else:  # No IP found. pylint: disable=useless-else-on-loop
        raise LookupError('Could not successfully retrieve ip of server.')


class Domain(object):
    """An interface for a Domain object.

    Used for binding an IP and a domain name in one object. Also provides a
    method for setting the IP using a DNS API. Note that this is yet to be
    implemented.

    :param name: The domain name the Domain() object represents.

    Usage::

        >>> from ext_pylib.domain import Domain
        >>> domain = Domain('example.com')
        >>> domain.name
        'example.com'
    """

    def __init__(self, name=''):
        """Initializes a new Domain instance."""
        self.name = name

    def __repr__(self):
        """Returns a python string that evaluates to the object instance."""
        return 'Domain("' + self.name + '")'

    def __str__(self):
        """Returns a string with the domain name."""
        return self.name

    def __add__(self, other):
        """Allows concatenation."""
        return str(self) + other

    def __radd__(self, other):
        """Allows concatenation."""
        return other + str(self)

    def __eq__(self, other):
        """Tests comparison (with a string or another Domain object.)"""
        try:
            return self.name == other.name
        except AttributeError:
            return self.name == other

    def set_ip(self, ip=''):
        """Points the domain's A record to this server -- not yet implemented."""
        # TODO: Implement DNS API
        print('Current serverip: ' + self.ip)
        print('[*] DNS API not yet implemented.')
        print('[*] Cannot set to IP: ' + ip)
        print('[*] DNS API not yet implemented.')

    def get_ip(self):
        """Gets the A Record IP of the domain and updates the cached copy."""
        self._ip = socket.gethostbyname(self.name)
        return self._ip

    @property
    def ip(self):
        """Returns the *cached* IP of the domain (or retrieves it if necessary)."""
        try:
            return self._ip
        except AttributeError:
            self._ip = self.get_ip()
            return self._ip

    @property
    def name(self):
        """Returns the domain name as a string."""
        return self._name

    @name.setter
    def name(self, name):
        """Validates and set the domain name."""
        if name in ['', None]:
            raise ValueError('Domain name cannot be an empty string.')
        if name[-1] == ".":
            name = name[:-1]  # strip exactly one dot from the right, if present
        if len(name) > 255:
            raise ValueError('Domain Name cannot be longer than 255 characters.')
        allowed_chars = re.compile(r"(?!-)[A-Z\d-]{1,63}(?<!-)$", re.IGNORECASE)
        if not all(allowed_chars.match(x) for x in name.split(".")):
            raise ValueError('Domain Name ' + name + ' is not valid.')
        self._name = name

    @property
    def domain(self):
        """Returns the domain name as a string. This is just an alias."""
        return self.name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# name:             password.py
# maintainer:       Harold Bradley III
# email:            harold@bradleystudio.net
#

"""
ext_pylib.password.password
~~~~~~~~~~~~~~~~~~~~~~~~~~~

A module to create passwords.
"""

from random import choice


# Defines the default characters for use in generating a password
DEFAULT_CHAR_SET = {
    'small': 'abcdefghijklmnopqrstuvwxyz',
    'nums': '0123456789',
    'big': 'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
    'special': '^!$%&=?{[]}+~#-_.:,;<>|'
}

def generate_pw(length=18, char_set=None):
    """Generates a pseudo-randomly generated password and returns it as a string.
    Adapted from: http://code.activestate.com/recipes/578169-extremely-strong-password-generator/"""
    char_set = char_set or DEFAULT_CHAR_SET
    password = []

    while len(password) < length:
        subset = choice(list(char_set.keys()))  # Get a random subset of characters
                                                # from which to choose
        # Ensure it isn't the same subset as the previous character in the password.
        if password and password[-1] in char_set[subset]:
            continue
        else:
            password.append(choice(char_set[subset]))

    return ''.join(password)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
ext_pylib.password
~~~~~~~~~~~~~~~~~~

Module for working with passwords.
"""

from __future__ import absolute_import

from .password import generate_pw



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# name:             colors.py
# author:           Harold Bradley III
# email:            harold@bradleystudio.net
# created on:       02/23/2016

"""
ext_pylib.terminal.colors
~~~~~~~~~~~~~~~~~~~~~~~~~

Functions for text effects and colors using ANSI escape sequences.
They return strings wrapped with the codes.
"""

from __future__ import absolute_import

from . import ansi


def sgr_wrapper(string, *args):
    """Takes a string and wraps an ANSI SGR escape sequence around it.

    If a reset is found in the middle of the string, the attribute is set again
    immediately following the reset. The entire string is concluded with the
    reset. This allows for unlimited nesting. (Without this, the first reset
    would clear the sequence for the entire rest of the string.)

    :param string: the string around which to wrap the ANSI SGR escape sequence.
    :param *args: one or more escape args with which to put in one escape
        sequence.

    Usage::

        >>> print bg_white(black('This section of text is ' + red(bold('important')) + '.'))
        This section of text is important.

    """
    escape = ansi.sgr(*args)
    reset = ansi.sgr(ansi.RESET)
    _string = ''
    for _segment in string.split(reset):
        _string = _string + escape + _segment + reset
    return _string

def sgr_combiner(string, attribute, *funcs, **additional):
    """Combines multiple sgr attributes into one string using sgr_wrapper.

    This is called by individual attribute functions and allows either chaining
    multiple functions or passing attribute functions as arguments of other
    attribute functions.

    :param string: the string around which to wrap the SGR codes.
    :param attribute: the ansi attribute to be used and combined with optional
        additional attributes.
    :param *funcs: optional attribute functions to be applied.
    :param **additional: additional attributes to be combined with attribute.
        This parameter is intended for internal use for recursion.

    Usage::

        >>> print(bold('The important string.' red, on_white))
        The important string.

        >>> print(bold(red(on_white('The important string.'))))
        The important string.

    Note that this allows for nonsensical combinations. There is no error
    checking, but a string cannot be both red and green, nor can it blink while
    being underlined. The last escape code is the one that applies. It is up to
    the user to make sure the attributes can be combined.
    """
    _attributes = additional['attributes'] + (attribute,) if 'attributes' in additional else (attribute,)
    if funcs:
        _additional = { 'attributes' : _attributes }
        _next_func, _funcs = funcs[0], funcs[1:]
        return _next_func(string, *_funcs, **_additional)
    else:
        return sgr_wrapper(string, *_attributes)


# Text effects

def normal(string, *funcs, **additional):
    """Text effect - normal. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.NORMAL, *funcs, **additional)

def underline(string, *funcs, **additional):
    """Text effect - underline. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.UNDERLINE, *funcs, **additional)

def bold(string, *funcs, **additional):
    """Text effect - bold. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.BOLD, *funcs, **additional)

def blink(string, *funcs, **additional):
    """Text effect - blink. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.BLINK, *funcs, **additional)

def rblink(string, *funcs, **additional):
    """Text effect - rblink. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.RBLINK, *funcs, **additional)

def reverse(string, *funcs, **additional):
    """Text effect - reverse. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.REVERSE, *funcs, **additional)

def conceal(string, *funcs, **additional):
    """Text effect - conceal. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.CONCEAL, *funcs, **additional)


# Basic colors

def black(string, *funcs, **additional):
    """Text color - black. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.BLACK, *funcs, **additional)

def red(string, *funcs, **additional):
    """Text color - red. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.RED, *funcs, **additional)

def green(string, *funcs, **additional):
    """Text color - green. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.GREEN, *funcs, **additional)

def yellow(string, *funcs, **additional):
    """Text color - yellow. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.YELLOW, *funcs, **additional)

def blue(string, *funcs, **additional):
    """Text color - blue. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.BLUE, *funcs, **additional)

def magenta(string, *funcs, **additional):
    """Text color - magenta. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.MAGENTA, *funcs, **additional)

def cyan(string, *funcs, **additional):
    """Text color - cyan. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.CYAN, *funcs, **additional)

def white(string, *funcs, **additional):
    """Text color - white. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.WHITE, *funcs, **additional)


# Basic background colors

def on_black(string, *funcs, **additional):
    """Text background color - black. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.BG_BLACK, *funcs, **additional)

def on_red(string, *funcs, **additional):
    """Text background color - red. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.BG_RED, *funcs, **additional)

def on_green(string, *funcs, **additional):
    """Text background color - green. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.BG_GREEN, *funcs, **additional)

def on_yellow(string, *funcs, **additional):
    """Text background color - yellow. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.BG_YELLOW, *funcs, **additional)

def on_blue(string, *funcs, **additional):
    """Text background color - blue. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.BG_BLUE, *funcs, **additional)

def on_magenta(string, *funcs, **additional):
    """Text background color - magenta. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.BG_MAGENTA, *funcs, **additional)

def on_cyan(string, *funcs, **additional):
    """Text background color - cyan. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.BG_CYAN, *funcs, **additional)

def on_white(string, *funcs, **additional):
    """Text background color - white. (see sgr_combiner())."""
    return sgr_combiner(string, ansi.BG_WHITE, *funcs, **additional)


# Colors on a black background

def red_on_black(string):
    """Text color - red on background color - black. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.RED, ansi.BG_BLACK)

def green_on_black(string):
    """Text color - green on background color - black. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.GREEN, ansi.BG_BLACK)

def yellow_on_black(string):
    """Text color - yellow on background color - black. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.YELLOW, ansi.BG_BLACK)

def blue_on_black(string):
    """Text color - blue on background color - black. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.BLUE, ansi.BG_BLACK)

def magenta_on_black(string):
    """Text color - magenta on background color - black. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.MAGENTA, ansi.BG_BLACK)

def cyan_on_black(string):
    """Text color - cyan on background color - black. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.CYAN, ansi.BG_BLACK)

def white_on_black(string):
    """Text color - white on background color - black. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.WHITE, ansi.BG_BLACK)


# Colors on a red background

def black_on_red(string):
    """Text color - black on background color - red. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.BLACK, ansi.BG_RED)

def green_on_red(string):
    """Text color - green on background color - red. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.GREEN, ansi.BG_RED)

def yellow_on_red(string):
    """Text color - yellow on background color - red. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.YELLOW, ansi.BG_RED)

def blue_on_red(string):
    """Text color - blue on background color - red. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.BLUE, ansi.BG_RED)

def magenta_on_red(string):
    """Text color - magenta on background color - red. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.MAGENTA, ansi.BG_RED)

def cyan_on_red(string):
    """Text color - cyan on background color - red. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.CYAN, ansi.BG_RED)

def white_on_red(string):
    """Text color - white on background color - red. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.WHITE, ansi.BG_RED)


# Colors on a green background

def black_on_green(string):
    """Text color - black on background color - green. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.BLACK, ansi.BG_GREEN)

def red_on_green(string):
    """Text color - red on background color - green. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.RED, ansi.BG_GREEN)

def yellow_on_green(string):
    """Text color - yellow on background color - green. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.YELLOW, ansi.BG_GREEN)

def blue_on_green(string):
    """Text color - blue on background color - green. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.BLUE, ansi.BG_GREEN)

def magenta_on_green(string):
    """Text color - magenta on background color - green. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.MAGENTA, ansi.BG_GREEN)

def cyan_on_green(string):
    """Text color - cyan on background color - green. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.CYAN, ansi.BG_GREEN)

def white_on_green(string):
    """Text color - white on background color - green. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.WHITE, ansi.BG_GREEN)


# Colors on a yellow background

def black_on_yellow(string):
    """Text color - black on background color - yellow. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.BLACK, ansi.BG_YELLOW)

def red_on_yellow(string):
    """Text color - red on background color - yellow. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.RED, ansi.BG_YELLOW)

def green_on_yellow(string):
    """Text color - green on background color - yellow. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.GREEN, ansi.BG_YELLOW)

def blue_on_yellow(string):
    """Text color - blue on background color - yellow. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.BLUE, ansi.BG_YELLOW)

def magenta_on_yellow(string):
    """Text color - magenta on background color - yellow. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.MAGENTA, ansi.BG_YELLOW)

def cyan_on_yellow(string):
    """Text color - cyan on background color - yellow. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.CYAN, ansi.BG_YELLOW)

def white_on_yellow(string):
    """Text color - white on background color - yellow. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.WHITE, ansi.BG_YELLOW)


# Colors on a blue background

def black_on_blue(string):
    """Text color - black on background color - blue. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.BLACK, ansi.BG_BLUE)

def red_on_blue(string):
    """Text color - red on background color - blue. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.RED, ansi.BG_BLUE)

def green_on_blue(string):
    """Text color - green on background color - blue. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.GREEN, ansi.BG_BLUE)

def yellow_on_blue(string):
    """Text color - yellow on background color - blue. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.YELLOW, ansi.BG_BLUE)

def magenta_on_blue(string):
    """Text color - magenta on background color - blue. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.MAGENTA, ansi.BG_BLUE)

def cyan_on_blue(string):
    """Text color - cyan on background color - blue. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.CYAN, ansi.BG_BLUE)

def white_on_blue(string):
    """Text color - white on background color - blue. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.WHITE, ansi.BG_BLUE)


# Colors on a magenta background

def black_on_magenta(string):
    """Text color - black on background color - magenta. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.BLACK, ansi.BG_MAGENTA)

def red_on_magenta(string):
    """Text color - red on background color - magenta. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.RED, ansi.BG_MAGENTA)

def green_on_magenta(string):
    """Text color - green on background color - magenta. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.GREEN, ansi.BG_MAGENTA)

def yellow_on_magenta(string):
    """Text color - yellow on background color - magenta. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.YELLOW, ansi.BG_MAGENTA)

def blue_on_magenta(string):
    """Text color - blue on background color - magenta. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.BLUE, ansi.BG_MAGENTA)

def cyan_on_magenta(string):
    """Text color - cyan on background color - magenta. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.CYAN, ansi.BG_MAGENTA)

def white_on_magenta(string):
    """Text color - white on background color - magenta. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.WHITE, ansi.BG_MAGENTA)


# Colors on a cyan background

def black_on_cyan(string):
    """Text color - black on background color - cyan. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.BLACK, ansi.BG_CYAN)

def red_on_cyan(string):
    """Text color - red on background color - cyan. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.RED, ansi.BG_CYAN)

def green_on_cyan(string):
    """Text color - green on background color - cyan. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.GREEN, ansi.BG_CYAN)

def yellow_on_cyan(string):
    """Text color - yellow on background color - cyan. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.YELLOW, ansi.BG_CYAN)

def blue_on_cyan(string):
    """Text color - blue on background color - cyan. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.BLUE, ansi.BG_CYAN)

def magenta_on_cyan(string):
    """Text color - magenta on background color - cyan. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.MAGENTA, ansi.BG_CYAN)

def white_on_cyan(string):
    """Text color - white on background color - cyan. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.WHITE, ansi.BG_CYAN)


# Colors on a white background

def black_on_white(string):
    """Text color - black on background color - white. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.BLACK, ansi.BG_WHITE)

def red_on_white(string):
    """Text color - red on background color - white. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.RED, ansi.BG_WHITE)

def green_on_white(string):
    """Text color - green on background color - white. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.GREEN, ansi.BG_WHITE)

def yellow_on_white(string):
    """Text color - yellow on background color - white. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.YELLOW, ansi.BG_WHITE)

def blue_on_white(string):
    """Text color - blue on background color - white. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.BLUE, ansi.BG_WHITE)

def magenta_on_white(string):
    """Text color - magenta on background color - white. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.MAGENTA, ansi.BG_WHITE)

def cyan_on_white(string):
    """Text color - cyan on background color - white. (see sgr_combiner())."""
    return sgr_wrapper(string, ansi.CYAN, ansi.BG_WHITE)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pylint: disable=bad-whitespace,redefined-builtin

"""
ext_pylib setup
"""


from os import path
from codecs import open
from setuptools import setup


with open('README.rst', 'r', 'utf-8') as f:
    README = f.read()

HERE = path.abspath(path.dirname(__file__))

setup(
    name = 'ext_pylib',

    version = '0.1',

    description = 'Extra python libraries for scaffolding server scripts.',
    long_description = README,

    # The project's main homepage.
    url = 'https://github.com/hbradleyiii/ext_pylib',
    download_url = 'https://github.com/hbradleyiii/ext_pylib/archive/0.1.tar.gz',

    # Author details
    author = 'Harold Bradley III',
    author_email = 'harold@bradleystudio.net',

    # Choose your license
    license = 'MIT License',

    # See https://pypi.python.org/pypi?%3Aaction = list_classifiers
    classifiers = [
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    # What does your project relate to?
    keywords = ['server development', 'files', 'domain names'],

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages = [
        'ext_pylib',
        'ext_pylib.domain',
        'ext_pylib.files',
        'ext_pylib.input',
        'ext_pylib.meta',
        'ext_pylib.password',
        'ext_pylib.terminal',
        'ext_pylib.user',
    ],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires = ['requests'],

    test_requires = ['pytest>=2.8.0', 'mock'],

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data = { '' : ['LICENSE'], },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    data_files = [ ],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points = { },
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# name:             interactive_test.py
# author:           Harold Bradley III
# email:            harold@bradleystudio.net
# created on:       02/26/2016
#

"""
ext_pylib.terminal.interactive_test
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

This module is just an interactive test for the ansi and colors modules.
"""

from __future__ import absolute_import

from timeit import timeit

from ext_pylib.terminal import ansi, colors

def main():
    """The main test suite."""
    ansi.reset()

    print('ext_pylib interactive terminal test')
    ansi.cursor_down(2)

    print('Testing colors...')
    print('Row 1:  | Row 2:   | ')
    print(colors.black_on_red('black'))
    print(colors.red('red'))
    print(colors.green('green'))
    print(colors.yellow('yellow'))
    print(colors.blue('blue'))
    print(colors.magenta('magenta'))
    print(colors.cyan('cyan'))
    print(colors.white('white'))

    ansi.cursor_up(8)
    ansi.cursor_right(10)

    print(colors.black_on_white('black'))
    ansi.cursor_right(10)
    print(colors.red_on_black('red'))
    ansi.cursor_right(10)
    print(colors.green_on_red('green'))
    ansi.cursor_right(10)
    print(colors.yellow_on_green('yellow'))
    ansi.cursor_right(10)
    print(colors.blue_on_yellow('blue'))
    ansi.cursor_right(10)
    print(colors.magenta_on_blue('magenta'))
    ansi.cursor_right(10)
    print(colors.cyan_on_magenta('cyan'))
    ansi.cursor_right(10)
    print(colors.white_on_cyan('white'))

    print('\n')
    print(colors.underline(colors.red('This is red text that is underlined')))
    print(colors.bold(colors.green('This is green text that is bold')))
    print(colors.reverse(colors.green_on_blue('This is green on blue text that is reversed')))

if __name__ == '__main__':
    print(timeit(main, number=1))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# name:             prompts.py
# author:           Harold Bradley III
# email:            harold@bradleystudio.net
# created on:       11/03/2015

"""
ext_pylib.input.prompts
~~~~~~~~~~~~~~~~~~~~~~~

Functions for displaying and handling prompts on the terminal.
"""

from __future__ import print_function, unicode_literals

from ext_pylib.input import INPUT


def prompt(prompt_text, default=True):
    """Displays a yes/no prompt and returns the response as bool."""
    default_text = " [y] " if default else " [n] "
    while True:
        print(type(INPUT))
        response = INPUT('[?] ' + prompt_text + default_text).lower()
        responses = {
            ''    : default,
            'y'   : True,
            'yes' : True,
            'n'   : False,
            'no'  : False,
        }
        if response in responses:
            return responses[response]
        else:
            print('Response not understood.')

def prompt_str(prompt_text, default_str=''):
    """Prompts for a string, returns the result."""
    add_text = ' '
    if default_str != '':
        add_text = ' [' + default_str + '] '

    response = INPUT('[?] ' + prompt_text + add_text)
    if response == '':
        response = default_str
    return response

def warn_prompt(prompt_text, required_response):
    """Warning prompt that prompts for a specific text (required_response).
    This is useful for potentially dangerous actions."""
    while True:
        response = INPUT("[!!] " + prompt_text + " [type n or '" + required_response + "'] ")
        if response == required_response:
            return True
        elif response.lower() in ['n', 'no']:
            return False
        else:
            print('Response not understood. Please type "n" or "required_response"')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#    /''         |''\     |    |
#    \.     |    |../     | *  |..
#    /   \/ T    |   \  / | |  |  \
#    \.../\.| __ |    \/  | |  |../
#  ###################/###############
#                    /

"""
ext_pylib
~~~~~~~~~

Extra python libraries for scaffolding server scripts.
"""

from __future__ import absolute_import

from . import domain
from . import files
from . import input  # pylint: disable=redefined-builtin
from . import password
from . import terminal
from . import user


__title__ = 'ext_pylib'
__version__ = '0.1'
__author__ = 'Harold Bradley III'
__license__ = 'MIT'
__copyright__ = 'Copyright 2015-2016 Harold Bradley III'

# Soli Deo gloria. <><
